// ----------------------------------------------------------------------- //
// _input.cpp	input stuff
//
// last edited:	2001-05-12 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// includes

#include "stdafx.h"
#include "dinput.h"

#include "_shared.h"
#include "_input.h"

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// variables

LPDIRECTINPUT8			lpdi;		// direct input object
LPDIRECTINPUTDEVICE8	lpdimouse;	// mouse device

int						_i_mousex = 0, _i_mousey = 0;
int						_i_mouseb = 0;

// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// functions



// initializes direct input

int _input_init						// returns 0 if failed
	( 
		HINSTANCE instance,			// instance of the application
		HWND hWnd					// window handle
	)
{
	HRESULT hr;						// for results

	// create direct input object
	if (FAILED(hr = DirectInput8Create( instance, DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&lpdi, NULL )))
		return 0;

	// create mouse device
	if (FAILED(hr = lpdi->CreateDevice(GUID_SysMouse, &lpdimouse, NULL )))
		return 0;

	if (FAILED(hr = lpdimouse->SetDataFormat( &c_dfDIMouse )))
		return 0;

	lpdimouse->SetCooperativeLevel( hWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE );

	DIPROPDWORD dipdw;
    // the header
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    // the data
    dipdw.dwData            = 16;

	lpdimouse->SetProperty(DIPROP_BUFFERSIZE, &dipdw.diph);

	_input_acquire();
 
	return 1;
} // _input_init




// updates mouse coordinates and buttons?

void _input_update()
{
	int		bDone = 0;
	HRESULT hr;

	_i_mousex = 0;
	_i_mousey = 0;

	while (!bDone) 
	{
        DIDEVICEOBJECTDATA od;
        DWORD dwElements = 1;   // number of items to be retrieved

		hr = lpdimouse->GetDeviceData(sizeof(DIDEVICEOBJECTDATA), &od, &dwElements, 0);
		if (FAILED(hr) || dwElements == 0)
		{
			if (hr == DIERR_INPUTLOST)
				_input_acquire();
			else
				return;
		}

		switch (od.dwOfs) 
        {
            // Mouse horizontal motion
            case DIMOFS_X: 
				_i_mousex += od.dwData;
				break;
            case DIMOFS_Y: 
				_i_mousey += od.dwData;
				break;
			case DIMOFS_BUTTON0:
				_i_mouseb = (_i_mouseb & 0xFE) | !!(od.dwData&0x80);
				break;
			case DIMOFS_BUTTON1:
				_i_mouseb = (_i_mouseb & 0xFD) | (od.dwData&0x80?2:0);
				break;
		}
	} 
}

void _input_acquire()
{
	if (lpdimouse)
		lpdimouse->Acquire();
}