// ----------------------------------------------------------------------- //
// _r_main.cpp	rendering stuff
//
// last edited:	2001-05-11 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// notes
/*

 - after _r_matrix_set2d( w, h ) was called, you can call _r_draw_pic etc. 
   with coordinates from 0, 0 through w,h .. where 0,0 is the bottom, left
   corner of the screen

 */
// ----------------------------------------------------------------------- //


// ----------------------------------------------------------------------- //
// includes

#include "stdafx.h"
#include "stdio.h"

#include "_render.h"
#include "states.h"

// ----------------------------------------------------------------------- //



// ----------------------------------------------------------------------- //
// variables

// vertex buffer
LPDIRECT3DVERTEXBUFFER8		vertBuf;

// vertex format: x,y,z + 2 texture sets
DWORD						dwFvf = D3DFVF_XYZ|D3DFVF_TEX2;

D3DXMATRIX					matWorld;
D3DXMATRIX					matView;
D3DXMATRIX					matProj;
D3DXMATRIX					matTemp;

// ----------------------------------------------------------------------- //

// initializes the rendering engine

int		_r_init						// returns a nonzero value if successful
	()
{
	_log_printf("_r_init()");

	D3DCAPS8		caps;

	lpd3dd->GetDeviceCaps( &caps );
	gv.maxprimitives = caps.MaxPrimitiveCount;

	// create vertex buffer
	lpd3dd->CreateVertexBuffer( gv.maxprimitives*3*sizeof(flexVert_t), 0, dwFvf, D3DPOOL_DEFAULT, &vertBuf);

	// filtering
	lpd3dd->SetTextureStageState( 0, D3DTSS_MINFILTER, FILTERMETHOD );
	lpd3dd->SetTextureStageState( 0, D3DTSS_MIPFILTER, FILTERMETHOD );
	lpd3dd->SetTextureStageState( 0, D3DTSS_MAGFILTER, FILTERMETHOD );

	_r_texture_load( "textures/common/notfound", MAX_TEXTURES-1 );

	return 1;
} // _r_init()



// releases everything

void	_r_deinit()
{
	_log_printf("_r_deinit()");

	SafeRelease(vertBuf);
	_r_texture_freeall();
} // _r_deinit



// set matrices for 3d drawings

void	_r_matrix_set3d()
{
	// find right, forward and up vectors
	_vector_subtract	( gv.viewTarget, gv.viewPos, gv.viewDir  );
	_vector_cross		( gv.viewUp,   gv.viewDir,  gv.viewRight );	
	_vector_cross		( gv.viewDir,  gv.viewRight,  gv.viewUp2   );	

	// normalize them
	_vector_normalize	( gv.viewDir   );
	_vector_normalize	( gv.viewUp2   );
	_vector_normalize	( gv.viewRight );

	// view-matrix
	D3DXMatrixLookAtLH( &matView, (D3DXVECTOR3*)gv.viewPos, (D3DXVECTOR3*)gv.viewTarget, (D3DXVECTOR3*)gv.viewUp2 );
	lpd3dd->SetTransform( D3DTS_VIEW, &matView );

	// projection
	D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/180.0f*gv.fov, -_aspect, 0.1f, 5000.0f );
	lpd3dd->SetTransform( D3DTS_PROJECTION, &matProj );

	// world-matrix ..  // FIXME: shoulnd that be 0,0,0 ? yep!
	D3DXMatrixTranslation( &matWorld, 0,0,0 );
//	D3DXMatrixTranslation( &matWorld, -gv.viewPos[0], -gv.viewPos[1], -gv.viewPos[2] );
	lpd3dd->SetTransform( D3DTS_WORLD, &matWorld );
}




// set matrices for 2d drawings

void	_r_matrix_set2d
	( 
		float width,				// width of screen
		float height				// height
	)
{
	// view-matrix
	D3DXMatrixIdentity( &matView );
	lpd3dd->SetTransform( D3DTS_VIEW, &matView );

	// projection matreix
	D3DXMatrixOrthoLH( &matProj, width, height, 0, 2 );
	lpd3dd->SetTransform( D3DTS_PROJECTION, &matProj );

	// worldmatrix
	D3DXMatrixTranslation	( &matWorld, -width/2, -height/2, 0 );
	D3DXMatrixScaling		( &matTemp, 1, -1, 1 );
	D3DXMatrixMultiply		( &matView, &matView, &matTemp );
	lpd3dd->SetTransform	( D3DTS_WORLD, &matWorld );
}




// draws a number of vertices

void	_r_draw_verts
	(	
		flexVert_t *verts,			// pointer to a buffer holding the vertices
		int count,					// number of vertices (default: 4)
		D3DPRIMITIVETYPE type		// how to draw them (default: D3DPT_TRIANGLEFAN )
	)
{
	flexVert_t	*pVertices;			// pointer to be filled by ->Lock
	int			primCount;			// number of primitives the call would need

	// calculate primitives count

	if (type == D3DPT_TRIANGLEFAN)
	{
		primCount = count-2;

		if (primCount>gv.maxprimitives)
			primCount = gv.maxprimitives;
	}
	else if (type == D3DPT_TRIANGLESTRIP)
	{
		primCount = count-1;
		if (primCount>gv.maxprimitives)
			primCount = gv.maxprimitives;
	}
	else if (type == D3DPT_TRIANGLELIST)
	{
		primCount = count/3;

		// if to many primitives
		if (primCount>gv.maxprimitives)
		{
			// draw as much as possible 
			_r_draw_verts( verts, gv.maxprimitives*3, type );

			// then draw the rest
			_r_draw_verts( &verts[gv.maxprimitives*3], count - gv.maxprimitives*3, type );

			return;
		}
	}

	// lock vertex buffer
	if( FAILED( vertBuf->Lock( 0, 0, (BYTE**)&pVertices, 0 ) ) )
		return;

	// copy vertices into buffer
	memcpy( pVertices, verts, sizeof(flexVert_t)*count );
	vertBuf->Unlock();

	// use this vertex buffer as stream
	lpd3dd->SetStreamSource( 0, vertBuf, sizeof(flexVert_t) );
	lpd3dd->SetVertexShader( dwFvf );

	// draw!
	lpd3dd->DrawPrimitive( type, 0, primCount );
} // _r_draw_verts




// draws a 2d picture

void	_r_draw_pic
	( 
		int texture,				// specifies the texture to draw
		float x1,					// screen coordinates
		float y1, 
		float x2, 
		float y2, 
		float u1,					// texture coordinates (default 0,0 - 1,1 )
		float v1, 
		float u2, 
		float v2 
	)
{
	flexVert_t	verts[4];			// the vertex buffer

	// set textures
	_r_texture_set( texture ); 

	// set coordinates
	_vertex_set( &verts[0], x1, y1, 1, u1, v1 );
	_vertex_set( &verts[1], x2, y1, 1, u2, v1 );
	_vertex_set( &verts[2], x2, y2, 1, u2, v2 );
	_vertex_set( &verts[3], x1, y2, 1, u1, v2 );

	// draw!
	_r_draw_verts( verts );
} // _r_draw_pic




// draws a sprite using billboarding

void	_r_draw_billboard
	( 
		vec3_t pos,					// position of particle
		vec2_t size,				// size of the particle
		int texture					// texture to use
	)
{
	flexVert_t	vVerts[4];			// the vertex buffer

	vec3_t	vup,vright;

 	_vector_scale( gv.viewUp2,	 size[1], vup		);
	_vector_scale( gv.viewRight, size[0], vright	);

	_vector_copy ( pos,					vVerts[0].xyz	);
	_vector_add  ( vVerts[0].xyz, vup,	vVerts[0].xyz	);

	_vector_copy ( pos,					vVerts[1].xyz	);
	_vector_add  ( vVerts[1].xyz, vright, vVerts[1].xyz	);
	_vector_add  ( vVerts[1].xyz, vup,	vVerts[1].xyz	);

	_vector_copy ( pos,					vVerts[2].xyz	);
	_vector_add  ( vVerts[2].xyz, vright, vVerts[2].xyz	);
	
	_vector_copy ( pos,					vVerts[3].xyz	);

	// relocate to center

 	_vector_scale( gv.viewUp2,	 -size[1]/2, vup		);
	_vector_scale( gv.viewRight, -size[0]/2, vright	);

	for ( int i=0; i<4; i++ )
	{
		_vector_add( vVerts[i].xyz, vup,	vVerts[i].xyz	);
		_vector_add( vVerts[i].xyz, vright,	vVerts[i].xyz	);
	}

	// set texture coordinates
	vVerts[0].uv[0][0]  = 0;	vVerts[0].uv[0][1]  = 0;
	vVerts[1].uv[0][0]  = 1;	vVerts[1].uv[0][1]  = 0;
	vVerts[2].uv[0][0]  = 1;	vVerts[2].uv[0][1]  = 1;
	vVerts[3].uv[0][0]  = 0;	vVerts[3].uv[0][1]  = 1;

	// texture
	_r_texture_set( texture );

	// draw!
	_r_draw_verts( vVerts );
} // _r_draw_billboard




// draws an object

void	_r_draw_object
	( 
		object_t *obj				// pointer to the object
	)
{
	D3DXMATRIX	matTemp2;			// temporary matrix
	D3DXMATRIX	matWorldTmp;		// matrix to rescue the world matrix to
	flexVert_t	*verts;				// pointer to vertices

	matWorldTmp = matWorld;			// save world matrix

	// no object given or object has no model
	if (!obj || !obj->model) return;

	// i know this could be done more efficently, but it does the job :)

	// rotate the object
	D3DXMatrixRotationYawPitchRoll( &matTemp2, obj->angles[0], 0, 0);
	D3DXMatrixMultiply( &matTemp, &matTemp2, &matWorld );

	D3DXMatrixRotationYawPitchRoll( &matTemp2, 0, obj->angles[1], 0);
	D3DXMatrixMultiply( &matTemp, &matTemp2, &matTemp );

	D3DXMatrixRotationYawPitchRoll( &matTemp2, 0, 0, obj->angles[2]);
	D3DXMatrixMultiply( &matTemp, &matTemp2, &matTemp );

	// TODO: scaling

	// and move
	D3DXMatrixTranslation( &matTemp2, obj->pos[0], obj->pos[1], obj->pos[2] );
	D3DXMatrixMultiply( &matTemp, &matTemp, &matTemp2 );

	// set world matrix accordingly
	lpd3dd->SetTransform( D3DTS_WORLD, &matTemp );

	// copy vertices
	verts = (flexVert_t*)malloc(sizeof(flexVert_t)*obj->model->nTris*3);

	for (int i=0; i<obj->model->nTris; i++)
		for (int j=0; j<3; j++)
			_vertex_copy(&obj->model->verts[obj->model->tris[i*3+j]], &verts[i*3+j] );

	// draw!
	_r_draw_verts( verts, obj->model->nTris*3, D3DPT_TRIANGLELIST );

	// clean up
	free(verts);
	matWorld = matWorldTmp;
	lpd3dd->SetTransform( D3DTS_WORLD, &matWorld ); // restore the matrix
} // _r_draw_object




// calls the 3d scene rendering function if set

void	_r_render_3d()
{
	if (worldstates[gv.state].render_3d)
		worldstates[gv.state].render_3d();
} // _r_render_3d



// calls the 2d scene rendering function if set

void	_r_render_2d()
{
	if (worldstates[gv.state].render_2d)
		worldstates[gv.state].render_2d();
} // _r_render_2d




// renders the complete scene

void	_r_render_scene()
{
	// increase framecounter
	gv.frame++;

	// clear screen
	lpd3dd->Clear( 0, NULL, D3DCLEAR_ZBUFFER|D3DCLEAR_TARGET , D3DCOLOR_XRGB(gv.bg_red, gv.bg_green, gv.bg_blue), 1.0f, 0L );
	
	lpd3dd->BeginScene();


	// render 3d

	_r_matrix_set3d();
	_r_render_3d();


	//render 2d

	_r_matrix_set2d( wndWidth*1.0f, wndHeight*1.0f );
	_r_render_2d();

	lpd3dd->EndScene();


	// show scene
	lpd3dd->Present( NULL, NULL, NULL, NULL );
} // _r_render_scene

// ----------------------------------------------------------------------- //
