#ifndef		_RENDER_INCLUDED
#define		_RENDER_INCLUDED

#include	"_shared.h"
#include	"_gfx.h"

#define		MAX_TEXTURES		2048
#define		MAX_MODELVERTICES	1024

// filtering methods

// nearest
//#define	FILTERMETHOD	D3DTEXF_POINT

// linear
//#define	FILTERMETHOD	D3DTEXF_LINEAR

// anisotropic
#define	FILTERMETHOD	D3DTEXF_ANISOTROPIC


typedef struct flexVert_s
{
	vec3_t		xyz;
	vec2_t		uv[2];
} flexVert_t;

typedef	struct _model_s
{
	flexVert_t	verts[MAX_MODELVERTICES];
	int			tris[MAX_MODELVERTICES];

	int			nVerts;
	int			nTris;
} model_t;

typedef struct _object_s
{
	model_t		*model;

	vec3_t		pos;
	vec3_t		angles;
	vec3_t		scale;
} object_t;

struct		texture
{
	LPDIRECT3DTEXTURE8	pTexture;
	char				szTextureName[64];
	char				szFileName[MAX_PATH+1];
	D3DXIMAGE_INFO		Info;
};

extern	texture	_r_textures[MAX_TEXTURES];

extern D3DXMATRIX		matWorld;
extern D3DXMATRIX		matView;
extern D3DXMATRIX		matProj;

int		_r_init();
void	_r_deinit();

void	_r_draw_pic( int texture, float x1, float y1, float x2, float y2, float u1 = 0, float v1 = 0, float u2 = 1, float v2 = 1 );
void	_r_draw_billboard( vec3_t pos, vec2_t size, int texture );
void	_r_draw_verts( flexVert_t *verts, int count = 4, D3DPRIMITIVETYPE type = D3DPT_TRIANGLEFAN );

void	_r_draw_object( object_t *obj );

void	_r_draw_char( float x, float y, float size, char c, int tex );
void	_r_draw_string( float x, float y, float size, char *c, int tex );
void	_r_draw_string_dist( float x, float y, float size, float dist, char *c, int tex );
void	_r_draw_string_var( float x, float y, float size, float *dist, char *c, int tex );

void	_r_render_scene();

int		_r_texture_find		(	char		*szName							);
int		_r_texture_load		(	const char	*szFile,	char *szName, int num, int type = 1	);
int		_r_texture_load		(	char		*szName											);
int		_r_texture_load		(	char		*szName,	int num								);
int		_r_texture_create	(	int height, int width,	char *szName						);
void	_r_texture_freeall	(	);
void	_r_texture_set		(	int			num,	 int stage = 0							);
void	_r_texture_filter	(	int			num												);

void	_vertex_set(flexVert_t *vert, float x, float y, float z, float u, float v, float lu = 0, float lv = 0 );
void	_vertex_copy(flexVert_t *in, flexVert_t *out );

void	_vertex_rotatevertices( flexVert_t *buf, int count, float yaw, float pitch = 0, float roll = 0 );
void	_vertex_translatevertices( flexVert_t *buf, int count, vec3_t v );

#endif