#ifndef		_SHARED_INCLUDED
#define		_SHARED_INCLUDED

#include	"windows.h"

// ----------------------------------------------------------------------- //
// window stuff

extern	HWND			hWnd;
extern	int				bExit;

extern	int				wndWidth, wndHeight;

#define SafeRelease(a) if (a) a->Release();

// ----------------------------------------------------------------------- //
// vector stuff

typedef float vec_t;
typedef vec_t vec2_t[2];
typedef vec_t vec3_t[3];
typedef vec_t vec4_t[4];
typedef vec_t vec5_t[5];

typedef struct _plane_s
{
	vec3_t	n;
	vec_t	d;
} plane_t;

float	_vector_length	(	const vec3_t v											);
int 	_vector_compare	(	const vec3_t v1,	const vec3_t v2						);
void	_vector_clear	(	vec3_t in												);

void	_vector_cross	(	const vec3_t v1,	const vec3_t v2,	vec3_t cross	);
float	_vector_dot		(	vec3_t v1,			vec3_t v2							);

void	_vector_subtract(	vec3_t va,			vec3_t vb,			vec3_t out		);
void	_vector_add		(	vec3_t va,			vec3_t vb,			vec3_t out		);
void	_vector_copy	(	vec3_t in,			vec3_t out							);

void	_vector_scale	(	vec3_t v,			vec_t scale,		vec3_t out		);
void	_vector_normalize(	vec3_t in,			vec3_t out = NULL					);

void	_vector_set		(	vec3_t out,			float x, float y, float z			);

plane_t	_plane_frompoints(	vec3_t a,			vec3_t b,			vec3_t c		);
void	_plane_line_intersect( vec3_t a,		vec3_t b,	plane_t p, vec3_t out	);

// ----------------------------------------------------------------------- //
// file stuff

typedef struct _fileentry_s			// pak-file entry
{
	char	filename[128];
	long	offset;
	long	size;
} fileentry_t;

typedef struct _file_s
{
	char	*buf;
	long	size;
	long	pos;
} file_t;

file_t	*_file_load( char *filename );
file_t	*_file_loadfromdisk( char *filename );
file_t	*_file_loadfrompak( char *filename );

void	_file_close( file_t *f );
int		_file_seek( file_t *f, long offset,	int	origin );
int		_file_eof( file_t *f );

int		_file_loadpak( char *filename );
void	_file_closepak();

int		_file_getc( file_t *file );
int		_file_geti( file_t *file );
void	_file_getpixel( file_t *file, char *r, char *g, char *b, char *a, char bpp = 32 );

int		_file_read( char *out, long size, int n, file_t *file );

// ----------------------------------------------------------------------- //
// world stuff

typedef struct _gvar_s
{
	float	dtime;
	float	time;
	float	timescale;
	long	frame;

	int		statecount;
	int		state;
	float	statetime;
	float	laststateswitchtime;

	int		bg_red, bg_green, bg_blue;

	int		maxprimitives;

	// kameradaten
	vec3_t	viewPos, viewTarget, viewUp;
	vec3_t	viewDir, viewRight, viewUp2;

	float	fov;
} gvar_t;

extern	gvar_t	gv;								// globale variablen

void	_world_init();
void	_world_deinit();
void	_world_update( float t );


// ----------------------------------------------------------------------- //
// other stuff

#define		PI				3.1415926535897932384626433832795f
#define		PI2				6.283185307179586476925286766559f
#define		SQRT2			1.4142135623730950488016887242097f
#define		SIN45			0.70710678118654752440084436210485f
#define		EQUAL_EPSILON	0.01f

void	_convert_path( char *path );

void _log_init();
void _log_deinit();
void _log_printf( char *fmt, ... );

#endif