// ----------------------------------------------------------------------- //
// earth.cpp	element 1 - earth
//
// last edited:	2001-05-16 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// include 

#include "windows.h"
#include "_window.h"
#include "stdio.h"
#include "math.h"

#include "_shared.h"
#include "_render.h"

#include "states.h"

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// variables

float			heightfield[65536];	// the height of the floor
float			drawsize = 40;		// view distance
flexVert_t		hfverts[6];			// temporary vertices

int				floortex;			// floor texture
int				ohflighttex;		// original lighting information
int				hflighttex;			// lighting information
int				hftex;				// texture that holds the hf.tga

char			*hflight;			// lighting

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// functions


// initializes earth

void init_earth()
{
	long			x, y;			// temporary xy
	D3DLOCKED_RECT	lr;				// for locking the texture
	char			*buf;			// also

	// load textures and heightfield
	floortex		= _r_texture_load( "textures/floor" );
	hftex			= _r_texture_load( "data/hf" );
	ohflighttex		= _r_texture_load( "data/hflight" );
	hflighttex		= _r_texture_create( 256, 256, "*lightmap1" );

	// interpret heightfield
	_r_textures[hftex].pTexture->LockRect( 0, &lr, NULL, 0 );

	buf = (char*)lr.pBits;

	for (x=0; x<256; x++)
		for (y=0; y<256; y++)
			heightfield[x+y*256] = buf[x*4+y*lr.Pitch]/256.0f*16.0f-4;

	_r_textures[hftex].pTexture->UnlockRect( 0 );
} // init_earth


extern vec3_t	lightningpos;
extern float	lightning;

// renders the earth

void render_earth()
{
	D3DLOCKED_RECT	lr1,lr2;				// for locking the lightmap
	int		xx = (int)gv.viewPos[0],		// offset in hf 
			yy = (int)gv.viewPos[2];

	float	texscale = 0.4f;			

	// set states
	lpd3dd->SetRenderState		( D3DRS_ALPHABLENDENABLE,	FALSE			);
	lpd3dd->SetRenderState		( D3DRS_FOGENABLE,			TRUE			);
	lpd3dd->SetRenderState		( D3DRS_ZENABLE,			D3DZB_TRUE		);
	lpd3dd->SetRenderState		( D3DRS_ZFUNC,				D3DCMP_LESSEQUAL);
	lpd3dd->SetRenderState		( D3DRS_CULLMODE,			D3DCULL_NONE	);
	lpd3dd->SetRenderState		( D3DRS_FOGENABLE,			TRUE			);
	lpd3dd->SetTextureStageState( 0, D3DTSS_COLOROP,		D3DTOP_ADD		);
	lpd3dd->SetTextureStageState( 1, D3DTSS_COLOROP,		D3DTOP_MODULATE2X );
	lpd3dd->SetRenderState		( D3DRS_ZWRITEENABLE,		TRUE		);

	_r_textures[ohflighttex].pTexture->LockRect( 0, &lr1, NULL, 0 );
	_r_textures[hflighttex].pTexture->LockRect( 0, &lr2, NULL, 0 );

	memcpy( lr2.pBits, lr1.pBits, _r_textures[hflighttex].Info.Height * lr2.Pitch );

	char *p = (char*)lr2.pBits;
	int lx,ly,l,lxx,lyy;

	int r = 64+rand()%64;

	// TODO: dynamic lighting here

	_r_textures[hflighttex].pTexture->UnlockRect( 0 );
	_r_textures[ohflighttex].pTexture->UnlockRect( 0 );

	_r_texture_filter(hflighttex);

	// set textures
	_r_texture_set( hflighttex, 1 );
	_r_texture_set( floortex );

	// this is an often used variable so precalc it
	int		halfdrawsize = (int)(drawsize/2);

	// draw it
	for (int x=-halfdrawsize; x<halfdrawsize; x++)
	for (int y=-halfdrawsize; y<halfdrawsize; y++)
	{
		int		i[4];				// temporary indices for current cell
		float	hf[4];				// temporary cell heights
		float	tex[4];				// temporary texture pos
		float	ltex[4];			// temporary lighttexture pos

		// get indices
		i[0] = (x+xx+0+128)%256+(y+yy+0+128)%256*256;
		i[1] = (x+xx+1+128)%256+(y+yy+0+128)%256*256;
		i[2] = (x+xx+1+128)%256+(y+yy+1+128)%256*256;
		i[3] = (x+xx+0+128)%256+(y+yy+1+128)%256*256;

		// get heights
		for (int n=0; n<4; n++)
			hf[n] = heightfield[i[n]];

		// texture coordinates
		tex[0] = (x+xx+0)*texscale;		tex[1] = (x+xx+1)*texscale;		// left-right
		tex[2] = (y+yy+0)*texscale;		tex[3] = (y+yy+1)*texscale;		// top-bottom

		// lighting texture coordinates
		ltex[0] = (x+xx+0+128.0f)/256.0f;
		ltex[1] = (x+xx+1+128.0f)/256.0f;
		ltex[2] = (y+yy+0+128.0f)/256.0f;
		ltex[3] = (y+yy+1+128.0f)/256.0f;

		// create vertices
		_vertex_set( &hfverts[0], x,   hf[0], y,		tex[0],  tex[2],	ltex[0], ltex[2] );
		_vertex_set( &hfverts[1], x+1, hf[1], y,		tex[1],	 tex[2],	ltex[1], ltex[2] );
		_vertex_set( &hfverts[2], x+1, hf[2], y+1,		tex[1],	 tex[3],	ltex[1], ltex[3] );
		_vertex_set( &hfverts[3], x,   hf[0], y,		tex[0],	 tex[2],	ltex[0], ltex[2] );
		_vertex_set( &hfverts[4], x+1, hf[2], y+1,		tex[1],  tex[3],	ltex[1], ltex[3] );
		_vertex_set( &hfverts[5], x,   hf[3], y+1,		tex[0],  tex[3],	ltex[0], ltex[3] );

		// move under the camera
		for (n=0; n<6; n++)
		{
			hfverts[n].xyz[0]+=xx;
			hfverts[n].xyz[2]+=yy;
		}

		// draw!
		_r_draw_verts(  hfverts, 6, D3DPT_TRIANGLELIST );
	}
} // render_earth


float getheight(float x, float y)
{
	int xx = x+128;
	int yy = y+128;

	float fx = fmod(x,1);
	float fy = fmod(y,1);

	int		i[4];				// temporary indices for current cell
	float	hf[4];				// temporary cell heights

	// get indices
	i[0] = (xx+0)%256+(yy+0)%256*256;
	i[1] = (xx+1)%256+(yy+0)%256*256;
	i[2] = (xx+1)%256+(yy+1)%256*256;
	i[3] = (xx+0)%256+(yy+1)%256*256;

	// get heights
	for (int n=0; n<4; n++)
		hf[n] = heightfield[i[n]];

	float ypx;
	float ypz;

	if (fx>fy)
	{
		ypx = hf[1] - hf[0]; 
		ypz = hf[2] - hf[1];
	}
	else
	{
		ypx = hf[2] - hf[3]; 
		ypz = hf[3] - hf[0];
	}

	return hf[0] + fx*ypx + fy*ypz;
}


// ----------------------------------------------------------------------- //
